@@ -5,8 +5,7 @@ textext
 =======
 
 :Author: Pauli Virtanen <pav@iki.fi>
-:Author: Robert Szalai <szalai@mm.bme.hu>
-:Date: 2008-01-19
+:Date: 2008-04-26
 :License: BSD
 
 Textext is an extension for Inkscape_ that allows adding
@@ -18,6 +17,9 @@ This brings some of the power of TeX typesetting to Inkscape.
 Textext was initially based on InkLaTeX_ written by Toru Araki,
 but is now rewritten.
 
+Thanks to Robert Szalai, Rafal Kolanski, Brian Clarke, and Florent Becker
+for contributions.
+
 .. note::
    Unfortunately, the TeX input dialog is modal. That is, you cannot
    do anything else with Inkscape while you are composing the LaTeX
@@ -41,19 +43,17 @@ but is now rewritten.
 
 #------------------------------------------------------------------------------
 
-__version__ = "0.3.3"
-__author__ = "Pauli Virtanen <pav@iki.fi>, Robert Szalai <szalai@mm.bme.hu>"
+__version__ = "0.4.4"
 __docformat__ = "restructuredtext en"
 
-import sys, os
+import sys, os, glob, traceback, platform
 sys.path.append('/usr/share/inkscape/extensions')
 sys.path.append(r'c:/Program Files/Inkscape/share/extensions')
 sys.path.append(os.path.dirname(__file__))
 
-import inkex45 as inkex
-import os, sys, tempfile, traceback, glob, re
-import xml.dom.ext.reader.Sax2, xml.sax.handler
-from xml.dom.NodeFilter import NodeFilter
+import inkex
+import os, sys, tempfile, traceback, glob, re, md5, copy
+from lxml import etree
 
 USE_GTK = False
 try:
@@ -71,16 +71,22 @@ try:
 except ImportError:
     pass
 
-USE_WINDOWS = ('win' in sys.platform)
+USE_WINDOWS = (platform.system() == "Windows")
 
-TEXTEXT_NS = "http://www.iki.fi/pav/software/textext/"
-SVG_NS = "http://www.w3.org/2000/svg"
-XLINK_NS = "http://www.w3.org/1999/xlink"
+TEXTEXT_NS = u"http://www.iki.fi/pav/software/textext/"
+SVG_NS = u"http://www.w3.org/2000/svg"
+XLINK_NS = u"http://www.w3.org/1999/xlink"
 
-ID_PREFIX = "textext-obj-"
+ID_PREFIX = "textext-"
+
+NSS = {
+    u'textext': TEXTEXT_NS,
+    u'svg': SVG_NS,
+    u'xlink': XLINK_NS,
+}
 
 #------------------------------------------------------------------------------
-# Inkscape plugin functionality & GUI
+# GUI
 #------------------------------------------------------------------------------
 
 if USE_GTK:
@@ -90,11 +96,14 @@ if USE_GTK:
             self.text = text
             self.preamble_file = preamble_file
             self.scale_factor = scale_factor
+            self.callback = None
     
-        def ask(self):
+        def ask(self, callback):
+            self.callback = callback
+            
             window = gtk.Window(gtk.WINDOW_TOPLEVEL)
             window.set_title("TeX Text")
-            window.set_default_size(600, 600)
+            window.set_default_size(600, 400)
     
             label1 = gtk.Label(u"Preamble file:")
             label2 = gtk.Label(u"Scale factor:")
@@ -103,17 +112,23 @@ if USE_GTK:
 
             if hasattr(gtk, 'FileChooserButton'):
                 self._preamble = gtk.FileChooserButton("...")
-                self._preamble.set_filename(self.preamble_file)
+                if os.path.exists(self.preamble_file):
+                    self._preamble.set_filename(self.preamble_file)
                 self._preamble.set_action(gtk.FILE_CHOOSER_ACTION_OPEN)
             else:
                 self._preamble = gtk.Entry()
                 self._preamble.set_text(self.preamble_file)
-
-            self._scale_adj = gtk.Adjustment(value=self.scale_factor,
-                                             lower=0.01, upper=100,
+            
+            self._scale_adj = gtk.Adjustment(lower=0.01, upper=100,
                                              step_incr=0.1, page_incr=1)
             self._scale = gtk.SpinButton(self._scale_adj, digits=2)
-
+            
+            if self.scale_factor is not None:
+                self._scale_adj.set_value(self.scale_factor)
+            else:
+                self._scale_adj.set_value(1.0)
+                self._scale.set_sensitive(False)
+            
             self._text = gtk.TextView()
             self._text.get_buffer().set_text(self.text)
 
@@ -122,7 +137,8 @@ if USE_GTK:
             sw.set_shadow_type(gtk.SHADOW_IN)
             sw.add(self._text)
             
-            ok = gtk.Button(stock=gtk.STOCK_OK)
+            self._ok = gtk.Button(stock=gtk.STOCK_OK)
+            self._cancel = gtk.Button(stock=gtk.STOCK_CANCEL)
     
             # layout
             table = gtk.Table(3, 2, False)
@@ -135,16 +151,28 @@ if USE_GTK:
     
             vbox = gtk.VBox(False, 5)
             vbox.pack_start(table)
-            vbox.pack_end(ok, expand=False)
+            
+            hbox = gtk.HButtonBox()
+            hbox.add(self._ok)
+            hbox.add(self._cancel)
+            hbox.set_layout(gtk.BUTTONBOX_SPREAD)
+            
+            vbox.pack_end(hbox, expand=False, fill=False)
     
             window.add(vbox)
     
             # signals
             window.connect("delete-event", self.cb_delete_event)
-            ok.connect("clicked", self.cb_ok)
+            window.connect("key-press-event", self.cb_key_press)
+            self._ok.connect("clicked", self.cb_ok)
+            self._cancel.connect("clicked", self.cb_cancel)
     
-            # run
+            # show
             window.show_all()
+            self._text.grab_focus()
+
+            # run
+            self._window = window
             gtk.main()
     
             return self.text, self.preamble_file, self.scale_factor
@@ -152,6 +180,17 @@ if USE_GTK:
         def cb_delete_event(self, widget, event, data=None):
             gtk.main_quit()
             return False
+
+        def cb_key_press(self, widget, event, data=None):
+            # ctrl+return clicks the ok button
+            if gtk.gdk.keyval_name(event.keyval) == 'Return' \
+                   and gtk.gdk.CONTROL_MASK & event.state:
+                self._ok.clicked()
+                return True
+            return False
+        
+        def cb_cancel(self, widget, data=None):
+            raise SystemExit(1)
         
         def cb_ok(self, widget, data=None):
             buf = self._text.get_buffer()
@@ -163,8 +202,36 @@ if USE_GTK:
                     self.preamble_file = ""
             else:
                 self.preamble_file = self._preamble.get_text()
+
+            if self.scale_factor is not None:
+                self.scale_factor = self._scale_adj.get_value()
+            
+            try:
+                self.callback(self.text, self.preamble_file, self.scale_factor)
+            except StandardError, e:
+                err_msg = traceback.format_exc()
+                dlg = gtk.Dialog("Textext Error", self._window, 
+                                 gtk.DIALOG_MODAL)
+                dlg.set_default_size(600, 400)
+                btn = dlg.add_button(gtk.STOCK_OK, gtk.RESPONSE_CLOSE)
+                btn.connect("clicked", lambda w, d=None: dlg.destroy())
+                msg = gtk.Label()
+                msg.set_markup("<b>Error occurred while converting text from Latex to SVG:</b>")
                 
-            self.scale_factor = self._scale_adj.get_value()
+                txtw = gtk.ScrolledWindow()
+                txtw.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
+                txtw.set_shadow_type(gtk.SHADOW_IN)
+                txt = gtk.TextView()
+                txt.set_editable(False)
+                txt.get_buffer().set_text(err_msg)
+                txtw.add(txt)
+                
+                dlg.vbox.pack_start(msg, expand=False, fill=True)
+                dlg.vbox.pack_start(txtw, expand=True, fill=True)
+                dlg.show_all()
+                dlg.run()
+                return False
+            
             gtk.main_quit()
             return False
 
@@ -175,8 +242,11 @@ elif USE_TK:
             self.text = text
             self.preamble_file = preamble_file
             self.scale_factor = scale_factor
+            self.callback = None
     
-        def ask(self):
+        def ask(self, callback):
+            self.callback = callback
+            
             root = Tk.Tk()
             
             self._frame = Tk.Frame(root)
@@ -195,7 +265,10 @@ elif USE_TK:
             label.pack(pady=2, padx=5, side="left", anchor="w")
             self._scale = Tk.Scale(box, orient="horizontal", from_=0.1, to=10, resolution=0.1)
             self._scale.pack(expand=True, fill="x", pady=2, padx=5, anchor="e")
-            self._scale.set(self.scale_factor)
+            if self.scale_factor is not None:
+                self._scale.set(self.scale_factor)
+            else:
+                self._scale.set(1.0)
             box.pack(fill="x", expand=True)
             
             label = Tk.Label(self._frame, text="Text:")
@@ -204,18 +277,29 @@ elif USE_TK:
             self._text = Tk.Text(self._frame)
             self._text.pack(expand=True, fill="both", pady=5, padx=5)
             self._text.insert(Tk.END, self.text)
+           
+            box = Tk.Frame(self._frame)
+            self._btn = Tk.Button(box, text="OK", command=self.cb_ok)
+            self._btn.pack(ipadx=30, ipady=4, pady=5, padx=5, side="left")
             
-            self._btn = Tk.Button(self._frame, text="OK", command=self.cb_ok)
-            self._btn.pack(ipadx=30, ipady=4, pady=5, padx=5)
+            self._cancel = Tk.Button(box, text="Cancel", command=self.cb_cancel)
+            self._cancel.pack(ipadx=30, ipady=4, pady=5, padx=5, side="right")
+
+            box.pack(expand=False)
             
             root.mainloop()
             
+            self.callback(self.text, self.preamble_file, self.scale_factor)
             return self.text, self.preamble_file, self.scale_factor
+
+        def cb_cancel(self):
+            raise SystemExit(1)
     
         def cb_ok(self):
             self.text = self._text.get(1.0, Tk.END)
             self.preamble_file = self._preamble.get()
-            self.scale_factor = self._scale.get()
+            if self.scale_factor is not None:
+                self.scale_factor = self._scale.get()
             self._frame.quit()
 
 else:
@@ -228,16 +312,21 @@ else:
 class TexText(inkex.Effect):
     def __init__(self):
         inkex.Effect.__init__(self)
+
+        self.settings = Settings()
+        
         self.OptionParser.add_option(
-            "-t", "--text", action="store", type="str",
+            "-t", "--text", action="store", type="string",
             dest="text", default=None)
         self.OptionParser.add_option(
-            "-p", "--preamble-file", action="store", type="str",
-            dest="preamble_file", default="header.inc")
+            "-p", "--preamble-file", action="store", type="string",
+            dest="preamble_file",
+            default=self.settings.get('preamble', str, ""))
         self.OptionParser.add_option(
             "-s", "--scale-factor", action="store", type="float",
-            dest="scale_factor", default=1.0)
-
+            dest="scale_factor",
+            default=self.settings.get('scale', float, 1.0))
+    
     def effect(self):
         """Perform the effect: create/modify TexText objects"""
         global CONVERTERS
@@ -251,7 +340,7 @@ class TexText(inkex.Effect):
                 conv_cls.available()
                 converter_cls = conv_cls
                 break
-            except RuntimeError, e:
+            except StandardError, e:
                 converter_errors.append("%s: %s" % (conv_cls.__name__, str(e)))
         
         if not converter_cls:
@@ -263,16 +352,35 @@ class TexText(inkex.Effect):
         
         # Ask for TeX code
         if self.options.text is None:
-            asker = AskText(text, preamble_file, 1.0)
-            text, preamble_file, scale_factor = asker.ask()
-        else:
-            text = self.options.text
-            preamble_file = self.options.preamble_file
-            scale_factor = self.options.scale_factor
+            # If there is a transform, scale in GUI will be ignored
+            if old_node is not None:
+                scale_factor = None
+            else:
+                scale_factor = self.options.scale_factor
 
+            if not preamble_file:
+                preamble_file = self.options.preamble_file
+
+            if not os.path.isfile(preamble_file):
+                preamble_file = ""
+            
+            asker = AskText(text, preamble_file, scale_factor)
+            asker.ask(lambda t, p, s: self.do_convert(t, p, s,
+                                                      converter_cls, old_node))
+        else:
+            self.do_convert(self.options.text,
+                            self.options.preamble_file,
+                            self.options.scale_factor, converter_cls, old_node)
+
+    def do_convert(self, text, preamble_file, scale_factor, converter_cls,
+                   old_node):
+        
         if not text:
             return
 
+        if isinstance(text, unicode):
+            text = text.encode('utf-8')
+        
         # Convert
         converter = converter_cls(self.document)
         try:
@@ -285,19 +393,40 @@ class TexText(inkex.Effect):
 
         # Insert into document
 
-        new_node.setAttributeNS(TEXTEXT_NS,
-                                'textext:text',
-                                text.encode('string-escape'))
-        
-        new_node.setAttributeNS(TEXTEXT_NS, 'textext:preamble',
-                                preamble_file.encode('string-escape'))
+        # -- Set textext attribs
+        new_node.attrib['{%s}text'%TEXTEXT_NS] = text.encode('string-escape')
+        new_node.attrib['{%s}preamble'%TEXTEXT_NS] = \
+                                       preamble_file.encode('string-escape')
 
-        if old_node and old_node.hasAttribute('transform'):
-            new_node.setAttribute('transform',
-                                  old_node.getAttribute('transform'))
+        # -- Copy transform
+        try:
+            # Note: the new node does *not* have the SVG namespace prefixes!
+            #       This caused some problems as Inkscape couldn't properly
+            #       handle both svg: and prefixless entries in the same file
+            #       in some cases.
+            new_node.attrib['transform'] = old_node.attrib['transform']
+        except (KeyError, IndexError, TypeError, AttributeError):
+            pass
+
+        try:
+            new_node.attrib['transform'] = old_node.attrib['{%s}transform'%SVG_NS]
+        except (KeyError, IndexError, TypeError, AttributeError):
+            pass
+
+        # -- Copy style
+        if old_node is not None:
+            self.copy_style(old_node, new_node)
         
+        # -- Replace
         self.replace_node(old_node, new_node)
 
+        # -- Save settings
+        if os.path.isfile(preamble_file):
+            self.settings.set('preamble', preamble_file)
+        if scale_factor is not None:
+            self.settings.set('scale', scale_factor)
+        self.settings.save()
+    
     def get_old(self):
         """
         Dig out LaTeX code and name of preamble file from old
@@ -308,47 +437,138 @@ class TexText(inkex.Effect):
 
         for i in self.options.ids:
             node = self.selected[i]
-            if node.tagName != 'g': continue
+            if node.tag != '{%s}g' % SVG_NS: continue
             
-            if node.hasAttributeNS(TEXTEXT_NS, 'text'):
+            if '{%s}text'%TEXTEXT_NS in node.attrib:
                 # starting from 0.2, use namespaces
                 return (node,
-                        node.getAttributeNS(TEXTEXT_NS, 'text').decode('string-escape'),
-                        node.getAttributeNS(TEXTEXT_NS, 'preamble').decode('string-escape'))
-            elif node.hasAttribute('textext'):
+                        node.attrib.get('{%s}text'%TEXTEXT_NS, '').decode('string-escape'),
+                        node.attrib.get('{%s}preamble'%TEXTEXT_NS, '').decode('string-escape'))
+            elif '{%s}text'%SVG_NS in node.attrib:
                 # < 0.2 backward compatibility
                 return (node,
-                        node.getAttribute('textext').decode('string-escape'),
-                        node.getAttribute('texpreamble').decode('string-escape'))
+                        node.attrib.get('{%s}text'%SVG_NS, '').decode('string-escape'),
+                        node.attrib.get('{%s}preamble'%SVG_NS, '').decode('string-escape'))
         return None, "", ""
 
-    def fix_xml_namespace(self, node):
-        """
-        Set the default XML namespace to http://www.w3.org/2000/svg
-        for the given node and all its children.
-
-        This is needed since pstoedit plot-svg generates namespaceless
-        SVG files.
-        """
-        node.setAttributeNS('http://www.w3.org/2000/xmlns/', 'xmlns', SVG_NS)
-        for c in node.childNodes:
-            if c.nodeType == c.ELEMENT_NODE:
-                self.fix_xml_namespace(c)
-
     def replace_node(self, old_node, new_node):
         """
         Replace an XML node old_node with new_node
         in self.document.
         """
-        new_node = self.document.importNode(new_node, True)
-        self.fix_xml_namespace(new_node)
         if old_node is None:
-            self.document.documentElement.appendChild(new_node)
+            self.current_layer.append(new_node)
         else:
-            parent = old_node.parentNode
-            parent.removeChild(old_node)
-            parent.appendChild(new_node)
+            parent = old_node.getparent()
+            parent.remove(old_node)
+            parent.append(new_node)
+
+
+    STYLE_ATTRS = ['fill','fill-opacity','fill-rule',
+                   'font-size-adjust','font-stretch',
+                   'font-style','font-variant',
+                   'font-weight','letter-spacing',
+                   'stroke','stroke-dasharray',
+                   'stroke-linecap','stroke-linejoin',
+                   'stroke-miterlimit','stroke-opacity',
+                   'text-anchor','word-spacing','style']
+    
+    def copy_style(self, old_node, new_node):
+        # XXX: Needs work...
+        #
+        #      We could try traversing the node tree downwards and
+        #      removing color-alteration from the attributes.
+        #      Not straightforward, need to read the SVG spec...
+        #
+        #      Removing style attributes does not work in general, because
+        #      at least pdf2svg relies on preserving the stroke attrs.
+        #
+        try:
+            new_node.attrib['style'] = old_node.attrib['style']
+        except (KeyError, IndexError, TypeError, AttributeError):
+            pass
+
+#------------------------------------------------------------------------------
+# Settings backend
+#------------------------------------------------------------------------------
+
+class Settings(object):
+    def __init__(self):
+        self.values = {}
         
+        if USE_WINDOWS:
+            self.keyname = r"Software\TexText\TexText"
+        else:
+            self.filename = os.path.expanduser("~/.inkscape/textextrc")
+
+        self.load()
+    
+    def load(self):
+        if USE_WINDOWS:
+            import _winreg
+            try:
+                key = _winreg.OpenKey(_winreg.HKEY_CURRENT_USER, self.keyname)
+            except:
+                return
+            try:
+                self.values = {}
+                for j in range(1000):
+                    try:
+                        name, data, dtype = _winreg.EnumValue(key, j)
+                    except EnvironmentError:
+                        break
+                    self.values[name] = str(data)
+            finally:
+                key.Close()
+        else:
+            try:
+                f = open(self.filename, 'r')
+            except (IOError, OSError):
+                return
+            try:
+                self.values = {}
+                for line in f.read().split("\n"):
+                    if not '=' in line: continue
+                    k, v = line.split("=", 1)
+                    self.values[k.strip()] = v.strip()
+            finally:
+                f.close()
+    
+    def save(self):
+        if USE_WINDOWS:
+            import _winreg
+            try:
+                key = _winreg.OpenKey(_winreg.HKEY_CURRENT_USER, self.keyname,
+                                      sam=_winreg.KEY_SET_VALUE | _winreg.KEY_WRITE)
+            except:
+                key = _winreg.CreateKey(_winreg.HKEY_CURRENT_USER, self.keyname)
+            try:
+                for k, v in self.values.iteritems():
+                    _winreg.SetValueEx(key, str(k), 0, _winreg.REG_SZ, str(v))
+            finally:
+                key.Close()
+        else:
+            d = os.path.dirname(self.filename)
+            if not os.path.isdir(d):
+                os.makedirs(d)
+            
+            f = open(self.filename, 'w')
+            try:
+                data = '\n'.join(["%s=%s" % (k,v)
+                                  for k,v in self.values.iteritems()])
+                f.write(data)
+            finally:
+                f.close()
+    
+    def get(self, key, typecast, default=None):
+        try:
+            return typecast(self.values[key])
+        except (KeyError, ValueError, TypeError):
+            return default
+
+    def set(self, key, value):
+        self.values[key] = str(value)
+
 
 #------------------------------------------------------------------------------
 # LaTeX converters
@@ -408,8 +628,9 @@ if USE_WINDOWS:
 
     program_files = os.environ.get('PROGRAMFILES')
     if program_files:
-        paths.append(os.path.join(program_files, 'pstoedit'))
-        paths.append(os.path.join(program_files, 'miktex 2.6', 'miktex', 'bin'))
+        paths += glob.glob(os.path.join(program_files, 'gs/gs*/bin'))
+        paths += glob.glob(os.path.join(program_files, 'pstoedit*'))
+        paths += glob.glob(os.path.join(program_files, 'miktex*/miktex/bin'))
         
     os.environ['PATH'] = os.path.pathsep.join(paths)
 
@@ -474,7 +695,7 @@ class LatexConverterBase(object):
         
         # Read preamble
         preamble = ""
-        if os.path.exists(preamble_file):
+        if os.path.isfile(preamble_file):
             f = open(preamble_file, 'r')
             preamble += f.read()
             f.close()
@@ -493,24 +714,19 @@ class LatexConverterBase(object):
         \end{document}
         """ % (preamble, latex_text)
         
-        cwd = os.getcwd()
-        os.chdir(self.tmp_path)
-        
         # Convert TeX to PDF
+
+        # Write tex
+        f_tex = open(self.tmp('tex'), 'w')
         try:
-            # Write tex
-            f_tex = open(self.tmp('tex'), 'w')
-            try:
-                f_tex.write(texwrapper)
-            finally:
-                f_tex.close()
-            
-            # Exec pdflatex: tex -> pdf
-            exec_command(['pdflatex', self.tmp('tex')] + latexOpts)
-            if not os.path.exists(self.tmp('pdf')):
-                raise RuntimeError("pdflatex didn't produce output")
+            f_tex.write(texwrapper)
         finally:
-            os.chdir(cwd)
+            f_tex.close()
+            
+        # Exec pdflatex: tex -> pdf
+        exec_command(['pdflatex', self.tmp('tex')] + latexOpts)
+        if not os.path.exists(self.tmp('pdf')):
+            raise RuntimeError("pdflatex didn't produce output")
 
     def remove_temp_files(self):
         """Remove temporary files"""
@@ -528,14 +744,20 @@ class LatexConverterBase(object):
 
 class PdfConverterBase(LatexConverterBase):
     def convert(self, latex_text, preamble_file, scale_factor):
-        self.tex_to_pdf(latex_text, preamble_file)
-        self.pdf_to_svg()
+        cwd = os.getcwd()
+        try:
+            os.chdir(self.tmp_path)
+            self.tex_to_pdf(latex_text, preamble_file)
+            self.pdf_to_svg()
+        finally:
+            os.chdir(cwd)
         
         new_node = self.svg_to_group()
-        if not new_node:
+        if new_node is None:
             return None
-        
-        new_node.setAttribute('transform', self.get_transform(scale_factor))
+
+        if scale_factor is not None:
+            new_node.attrib['transform'] = self.get_transform(scale_factor)
         return new_node
 
     def pdf_to_svg(self):
@@ -546,46 +768,35 @@ class PdfConverterBase(LatexConverterBase):
         """Get a suitable default value for the transform attribute"""
         raise NotImplementedError
 
-    def parse_xml(self, fn):
-        svg_stream = open(fn, 'r')
-        try:
-            reader_tex = xml.dom.ext.reader.Sax2.Reader()
-
-            # Prevent internet connections
-            reader_tex.parser.setFeature(xml.sax.handler.feature_external_ges,
-                                         False)
-            reader_tex.parser.setFeature(xml.sax.handler.feature_external_pes,
-                                         False)
-            reader_tex.parser.setFeature(xml.sax.handler.feature_validation,
-                                         False)
-
-            # Parse
-            return reader_tex.fromStream(svg_stream)
-        finally:
-            svg_stream.close()
-
     def svg_to_group(self):
         """
         Convert the SVG file to an SVG group node.
 
         :Returns: <svg:g> node
         """
-
-        # create xml.dom representation of the TeX file
-        doc_tex = self.parse_xml(self.tmp('svg'))
-        docel = doc_tex.documentElement
-        
-        # get latex paths from svg_out
+        tree = etree.parse(self.tmp('svg'))
+        self.fix_xml_namespace(tree.getroot())
         try:
-            return docel.getElementsByTagName('g')[0]
+            return copy.copy(tree.getroot().xpath('g')[0])
         except IndexError:
             return None
 
-if USE_WINDOWS:
-    PSTOEDIT_OK_RETURNCODE = 0
-else:
-    PSTOEDIT_OK_RETURNCODE = 1
-            
+    def fix_xml_namespace(self, node):
+        svg = '{%s}' % SVG_NS
+        
+        if node.tag.startswith(svg):
+            node.tag = node.tag[len(svg):]
+        
+        for key in node.attrib.keys():
+            if key.startswith(svg):
+                new_key = key[len(svg):]
+                node.attrib[new_key] = node.attrib[key]
+                del node.attrib[key]
+        
+        for c in node:
+            self.fix_xml_namespace(c)
+    
+
 class SkConvert(PdfConverterBase):
     """
     Convert PDF -> SK -> SVG using pstoedit and skconvert
@@ -612,7 +823,7 @@ class SkConvert(PdfConverterBase):
 
     def available(cls):
         """Check whether skconvert and pstoedit are available"""
-        out = exec_command(['pstoedit'], ok_return_value=PSTOEDIT_OK_RETURNCODE)
+        out = exec_command(['pstoedit'], ok_return_value=None)
         if 'version 3.44' in out and 'Ubuntu' in out:
             raise RuntimeError("Pstoedit version 3.44 on Ubuntu found, but it "
                                "contains too many bugs to be usable")
@@ -642,11 +853,12 @@ class PstoeditPlotSvg(PdfConverterBase):
     def available(cls):
         """Check whether pstoedit has plot-svg available"""
         out = exec_command(['pstoedit', '-help'],
-                           ok_return_value=PSTOEDIT_OK_RETURNCODE)
-        if 'version 3.44' in out and 'Ubuntu':
+                           ok_return_value=None)
+        if 'version 3.44' in out and 'Ubuntu' in out:
             raise RuntimeError("Pstoedit version 3.44 on Ubuntu found, but it "
                                "contains too many bugs to be usable")
-            return 'plot-svg' in out
+        if 'plot-svg' not in out:
+            raise RuntimeError("Pstoedit not compiled with plot-svg support")
     available = classmethod(available)
 
 class Pdf2Svg(PdfConverterBase):
@@ -655,29 +867,12 @@ class Pdf2Svg(PdfConverterBase):
     """
     def __init__(self, document):
         PdfConverterBase.__init__(self, document)
-        self.id_start = self.find_glyph_id_limits(document)
+        self.hash = None
 
-    def find_glyph_id_limits(self, document):
-        """
-        Find the first free id in the document
-        """
-        walker = document.createTreeWalker(document.documentElement,
-                                           NodeFilter.SHOW_ELEMENT, None, 0)
-        startid = 0
-
-        while True:
-            if walker.currentNode.hasAttribute('id'):
-                curid = walker.currentNode.getAttribute('id')
-                if curid.startswith(ID_PREFIX):
-                    try:
-                        startid = max(startid, int(curid[len(ID_PREFIX):]))
-                    except ValueError:
-                        pass # not a glyph of ours, skip it
-
-            if not walker.nextNode():
-                break
-
-        return startid
+    def convert(self, *a, **kw):
+        # compute hash for generating unique ids for sub-elements
+        self.hash = md5.new('%s%s' % (a, kw)).hexdigest()[:8]
+        return PdfConverterBase.convert(self, *a, **kw)
 
     def pdf_to_svg(self):
         exec_command(['pdf2svg', self.tmp('pdf'), self.tmp('svg'), '1'])
@@ -687,52 +882,40 @@ class Pdf2Svg(PdfConverterBase):
 
     def svg_to_group(self):
         # create xml.dom representation of the TeX file
-        doc_tex = self.parse_xml(self.tmp('svg'))
-        docel = doc_tex.documentElement
+        tree = etree.parse(self.tmp('svg'))
+        root = tree.getroot()
+        self.fix_xml_namespace(root)
 
         href_map = {}
 
         # Map items to new ids
-        walker = doc_tex.createTreeWalker(docel, NodeFilter.SHOW_ELEMENT, None, 0)
-        while True:
-            if walker.currentNode.hasAttribute('id'):
-                curid = walker.currentNode.getAttribute('id')
-
-                self.id_start += 1
-                new_id = "%s%d" % (ID_PREFIX, self.id_start)
-                
-                href_map['#' + curid] = '#' + new_id
-                walker.currentNode.setAttribute('id', new_id)
-
-            if not walker.nextNode():
-                break
+        for i, el in enumerate(root.xpath('//*[attribute::id]')):
+            cur_id = el.attrib['id']
+            new_id = "%s%s-%d" % (ID_PREFIX, self.hash, i)
+            href_map['#' + cur_id] = "#" + new_id
+            el.attrib['id'] = new_id
 
         # Replace hrefs
         url_re = re.compile('^url\((.*)\)$')
-        
-        walker = doc_tex.createTreeWalker(docel, NodeFilter.SHOW_ELEMENT, None, 0)
-        while True:
-            if walker.currentNode.hasAttributeNS(XLINK_NS, 'href'):
-                curid = walker.currentNode.getAttributeNS(XLINK_NS, 'href')
-                walker.currentNode.setAttributeNS(XLINK_NS, 'href',
-                                                  href_map.get(curid, curid))
 
-            if walker.currentNode.hasAttribute('clip-path'):
-                value = walker.currentNode.getAttribute('clip-path')
-                m = url_re.match(value)
-                if m:
-                    walker.currentNode.setAttribute(
-                        'clip-path',
-                        'url(%s)' % href_map.get(m.group(1), m.group(1)))
+        for el in root.xpath('//*[attribute::xlink:href]', namespaces=NSS):
+            href = el.attrib['{%s}href'%XLINK_NS]
+            el.attrib['{%s}href'%XLINK_NS] = href_map.get(href, href)
+
+        for el in root.xpath('//*[attribute::svg:clip-path]', namespaces=NSS):
+            value = el.attrib['clip-path']
+            m = url_re.match(value)
+            if m:
+                el.attrib['clip-path'] = \
+                    'url(%s)' % href_map.get(m.group(1), m.group(1))
 
-            if not walker.nextNode():
-                break
-        
         # Bundle everything in a single group
-        master_group = doc_tex.createElementNS(SVG_NS, 'g')
-        for c in list(doc_tex.documentElement.childNodes):
-            master_group.appendChild(c)
-        return master_group
+        master_group = etree.SubElement(root, 'g')
+        for c in root:
+            if c is master_group: continue
+            master_group.append(c)
+
+        return copy.copy(master_group)
 
     def available(cls):
         """Check whether pdf2svg is available, raise RuntimeError if not"""
